@@ -6,8 +6,6 @@ import socket
 import getpass
 import warnings
 import logging
-import time
-import traceback
 from select import select
 
 # 3rd Party
@@ -15,10 +13,6 @@ import paramiko
 from cryptography import utils
 from paramiko.ssh_exception import (
     SSHException,
-    ChannelException,
-    BadHostKeyException,
-    AuthenticationException,
-    BadAuthenticationType,
     PasswordRequiredException,
 )
 
@@ -61,12 +55,19 @@ class SSHShell:
         """
         self._session = self.session_open()
 
+    def socket_open(self):
+        """ open a socket to remote host
+            :return sock: socket object
+        """
+        sock = socket.create_connection((self.host, 22), 10)
+        return sock
+
     def session_open(self):
         """ opens a paramiko.SSHClient instance
             handles both key and password based authentication
             :return: paramiko.SSHClient instance
         """
-        logger.debug("entering session_open()")
+        logger.info("entering session_open()")
         kwargs = {"hostname": self.host, "username": self.user}
         ssh_client = paramiko.SSHClient()
         ssh_client.load_system_host_keys()
@@ -90,9 +91,18 @@ class SSHShell:
                 self._sock = paramiko.proxy.ProxyCommand(config.get("proxycommand"))
                 kwargs.update({"sock": self._sock})
 
+        if not self._sock:
+            try:
+                self._sock = self.socket_open()
+                kwargs.update({"sock": self._sock})
+            except (socket.gaierror, socket.herror):
+                raise ConnectionError("address or hostname not reachable")
+            except (socket.timeout, ConnectionRefusedError, IOError, OSError):
+                raise ConnectionError("error connecting to remote host on port 22")
+
         agent = paramiko.Agent()
         agent_keys = agent.get_keys()
-        logger.debug("ssh agent has {} keys".format(len(agent_keys)))
+        logger.info("ssh agent has {} keys".format(len(agent_keys)))
 
         if self.passwd is not None:
             kwargs.update(
@@ -115,7 +125,7 @@ class SSHShell:
                     "password": None,
                 }
             )
-            # paramiko is a little broken (see github issue #1664) 
+            # paramiko is a little broken (see github issue #1664)
             # work around by always asking for passphrase here
             # else "SSHException: encountered RSA key, expected OPENSSH key" error
             # when key has passphrase
@@ -125,7 +135,9 @@ class SSHShell:
             if passphrase != "":
                 kwargs.update({"passphrase": passphrase})
         elif len(agent_keys) == 0 and not key_found:
-            print("no ssh keys found, nor ssh agent running, skipping publickey ssh auth")
+            print(
+                "no ssh keys found, nor ssh agent running, skipping publickey ssh auth"
+            )
             kwargs.update({"allow_agent": False, "look_for_keys": False})
             ask_pass = True
 
@@ -191,7 +203,7 @@ class SSHShell:
             :type: string
         """
         self._chan.send("{}\n".format(cmd))
-        logger.debug("sent '{}'".format(cmd))
+        logger.info("sent '{}'".format(cmd))
 
     def close(self):
         """ terminates both the channel (if present) and the underlying session
@@ -216,14 +228,17 @@ class SSHShell:
             :type: string
         """
         result = False
-        self.write(cmd)
-        stdout = self.stdout_read(timeout)
-
-        if exitcode:
-            self.write("echo $?".format(cmd))
-            rc = self.stdout_read(timeout)
-            if re.search(r"\r\n0\r\n", rc, re.MULTILINE):
+        stdout = None
+        try:
+            self.write(cmd)
+            stdout = self.stdout_read(timeout)
+            if exitcode:
+                self.write("echo $?".format(cmd))
+                rc = self.stdout_read(timeout)
+                if re.search(r"\r\n0\r\n", rc, re.MULTILINE):
+                    result = True
+            elif stdout is not None and stdout != "":
                 result = True
-        elif stdout is not None and stdout != "":
-            result = True
+        except TimeoutError:
+            pass
         return result, stdout
