@@ -13,6 +13,7 @@ try:
 except ImportError:
     raise RuntimeError("Splitcopy requires Python 3.4+")
 import argparse
+import concurrent.futures
 import datetime
 import fnmatch
 import functools
@@ -20,7 +21,6 @@ import getpass
 import glob
 import hashlib
 import logging
-import multiprocessing
 import os
 import re
 import signal
@@ -31,6 +31,7 @@ import tempfile
 import time
 import traceback
 from contextlib import contextmanager
+from math import ceil
 
 # 3rd party
 from scp import SCPClient
@@ -40,7 +41,6 @@ from splitcopy.paramikoshell import SSHShell
 from splitcopy.progress import Progress
 from splitcopy.ftp import FTP
 
-_BUF_SIZE_SPLIT = 10240
 _BUF_SIZE_READ = 131072
 _BUF_SIZE = 1024
 
@@ -84,7 +84,7 @@ def main():
     args = parser.parse_args()
 
     if not args.log:
-        loglevel = "ERROR"
+        loglevel = "WARNING"
     else:
         loglevel = args.log[0]
 
@@ -126,6 +126,11 @@ def main():
         remote_path = source.split(":")[1]
         remote_file = os.path.basename(remote_path)
         remote_dir = os.path.dirname(remote_path)
+        if remote_dir == "" or remote_dir == ".":
+            remote_dir = "~"
+            remote_path = "{}/{}".format(remote_dir, remote_file)
+        if not remote_file:
+            raise SystemExit("src path doesn't specify a file name")
         get = True
     elif os.path.isfile(source):
         local_path = os.path.abspath(os.path.expanduser(source))
@@ -140,7 +145,7 @@ def main():
         local_file = os.path.basename(local_path)
         local_dir = os.path.dirname(local_path)
         file_size = os.path.getsize(local_path)
-        logger.debug("src file size is {}".format(file_size))
+        logger.info("src file size is {}".format(file_size))
     else:
         raise SystemExit(
             "specified source is not a valid path to a local "
@@ -203,7 +208,7 @@ def main():
             copy_proto = "scp"
     else:
         copy_proto = "scp"
-    logger.debug("copy_proto == {}".format(copy_proto))
+    logger.info("copy_proto == {}".format(copy_proto))
 
     if args.ssh_key is not None:
         ssh_key = os.path.abspath(args.ssh_key[0])
@@ -226,7 +231,7 @@ def main():
         "get": get,
         "noverify": noverify,
     }
-    logger.debug(kwargs)
+    logger.info(kwargs)
 
     splitcopy = SplitCopy(**kwargs)
 
@@ -249,7 +254,7 @@ def ftp_port_check(host):
         :returns: None
         :raises: any exception
     """
-    logger.debug("entering ftp_port_check()")
+    logger.info("entering ftp_port_check()")
     try:
         with socket.create_connection((host, 21), 10) as ftp_sock:
             pass
@@ -299,8 +304,10 @@ class SplitCopy:
         self.bsd_version = 0.0
         self.sshd_version = 0.0
         self.sha_bin = None
+        self.sha_len = None
         self.mute = False
-        self.sha_size = None
+        self.sha_hash = {}
+        self.executor = None
         self.ssh_kwargs = {
             "user": self.user,
             "host": self.host,
@@ -347,7 +354,7 @@ class SplitCopy:
         self.which_os()
 
         # ensure dest path is valid
-        self.validate_remote_path()
+        self.validate_remote_path_put()
 
         # check required binaries exist on remote host
         self.req_binaries()
@@ -384,8 +391,10 @@ class SplitCopy:
             sfiles = []
             for sfile in os.listdir("."):
                 if fnmatch.fnmatch(sfile, "{}*".format(self.local_file)):
-                    sfiles.append(sfile)
-            logger.debug("# of chunks = {}".format(len(sfiles)))
+                    sfiles.append([sfile, os.stat(sfile).st_size])
+            if not len(sfiles):
+                self.close(err_str="file split operation failed")
+            logger.info("# of chunks = {}".format(len(sfiles)))
 
             # end of pre transfer checks, create tmp directory
             self.mkdir_remote()
@@ -413,7 +422,7 @@ class SplitCopy:
             loop = asyncio.get_event_loop()
             for sfile in sfiles:
                 task = loop.run_in_executor(
-                    None, functools.partial(self.put_files, sfile)
+                    self.executor, functools.partial(self.put_files, sfile)
                 )
                 self.tasks.append(task)
             try:
@@ -473,10 +482,8 @@ class SplitCopy:
         # determine remote host os
         self.which_os()
 
-        # begin pre transfer checks, check if remote file exists
-        result, stdout = self.ss.run("test -r {}".format(self.remote_path))
-        if not result:
-            self.close(err_str="file on remote host is not readable - does it exist?")
+        # ensure dest path is valid
+        self.validate_remote_path_get()
 
         # check required binaries exist on remote host
         self.req_binaries()
@@ -515,15 +522,18 @@ class SplitCopy:
         self.split_file_remote()
 
         # add chunk names to a list
-        result, stdout = self.ss.run("ls -1 {}/".format(self.remote_tmpdir))
+        result, stdout = self.ss.run("ls -l {}/".format(self.remote_tmpdir))
         if not result:
             self.close(err_str="couldn't get list of files from host")
         remote_files = stdout.split("\r\n")
         sfiles = []
         for sfile in remote_files:
-            if fnmatch.fnmatch(sfile, "{}*".format(self.remote_file)):
-                sfiles.append(sfile)
-        logger.debug("# of chunks = {}".format(len(sfiles)))
+            if fnmatch.fnmatch(sfile, "* {}*".format(self.remote_file)):
+                sfile = sfile.split()
+                sfiles.append([sfile[-1], sfile[-5]])
+        if not len(sfiles):
+            self.close(err_str="file split operation failed")
+        logger.info("# of chunks = {}".format(len(sfiles)))
 
         # begin connection/rate limit check and transfer process
         if self.junos or self.evo:
@@ -549,7 +559,7 @@ class SplitCopy:
             self.tasks = []
             for sfile in sfiles:
                 task = loop.run_in_executor(
-                    None, functools.partial(self.get_files, sfile)
+                    self.executor, functools.partial(self.get_files, sfile)
                 )
                 self.tasks.append(task)
             print("starting transfer...")
@@ -598,7 +608,7 @@ class SplitCopy:
             no support for Windows OS running OpenSSH
             :returns None:
         """
-        logger.debug("entering which_os()")
+        logger.info("entering which_os()")
         self.ss.run("start shell", exitcode=False)
         result, stdout = self.ss.run("uname")
         if not result:
@@ -609,17 +619,59 @@ class SplitCopy:
             self.evo = True
         else:
             self.junos_os()
-        logger.debug(
+        logger.info(
             "evo = {}, junos = {}, bsd_version = {}, sshd_version = {}".format(
                 self.evo, self.junos, self.bsd_version, self.sshd_version
             )
         )
 
-    def validate_remote_path(self):
+    def validate_remote_path_get(self):
+        """ path must be a full path, expand as required
+            :return: None
+        """
+        logger.info("entering validate_remote_path_get()")
+        if re.match(r"~", self.remote_dir):
+            result, stdout = self.ss.run("ls -d {}".format(self.remote_dir))
+            if result:
+                self.remote_dir = stdout.split("\n")[1].rstrip()
+                self.remote_path = "{}/{}".format(self.remote_dir, self.remote_file)
+                logger.info(
+                    "remote_dir now = {}, remote_file now = {}".format(
+                        self.remote_dir, self.remote_file
+                    )
+                )
+            else:
+                self.close(
+                    err_str="unable to expand remote path {}".format(self.remote_path)
+                )
+
+        # bail if its a directory
+        result, stdout = self.ss.run("test -d {}".format(self.remote_path))
+        if result:
+            self.close(err_str="src path is a directory, not a file")
+
+        # begin pre transfer checks, check if remote file exists
+        result, stdout = self.ss.run("test -r {}".format(self.remote_path))
+        if not result:
+            self.close(err_str="file on remote host is not readable")
+
+        # is it a symlink? if so, rewrite remote_path with linked file path
+        result, stdout = self.ss.run("test -L {}".format(self.remote_path))
+        if result:
+            logger.info("file is a symlink")
+            result, stdout = self.ss.run("ls -l {}".format(self.remote_path))
+            if result:
+                self.remote_path = stdout.split()[-2].rstrip()
+                self.remote_dir = os.path.dirname(self.remote_path)
+                self.remote_file = os.path.basename(self.remote_path)
+            else:
+                self.close(err_str="file on remote host is a symlink, failed to retrieve details")
+
+    def validate_remote_path_put(self):
         """ path provided can be a directory, a new or existing file
             :return: None
         """
-        logger.debug("entering validate_remote_path()")
+        logger.info("entering validate_remote_path_put()")
         if self.ss.run("test -d {}".format(self.remote_path))[0]:
             # target path provided is a directory
             self.remote_file = self.local_file
@@ -636,7 +688,7 @@ class SplitCopy:
                 # target path provided was a full path, file name matches src
                 self.remote_file = self.local_file
             self.remote_dir = os.path.dirname(self.remote_path)
-            logger.debug(
+            logger.info(
                 "self.remote_path = {}, self.remote_dir = {}, self.remote_file = {}".format(
                     self.remote_path, self.remote_dir, self.remote_file
                 )
@@ -653,7 +705,7 @@ class SplitCopy:
             :returns result: True if OS is EVO
             :type: boolean
         """
-        logger.debug("entering evo_os()")
+        logger.info("entering evo_os()")
         result, stdout = self.ss.run("test -e /usr/sbin/evo-pfemand")
         return result
 
@@ -661,7 +713,7 @@ class SplitCopy:
         """ determines if host is JUNOS
             :returns None:
         """
-        logger.debug("entering junos_os()")
+        logger.info("entering junos_os()")
         result, stdout = self.ss.run("uname -i")
         if not result:
             self.close(err_str="failed to determine remote host os")
@@ -681,7 +733,7 @@ class SplitCopy:
         """ determines the BSD version of JUNOS
             :returns None:
         """
-        logger.debug("entering which_bsd()")
+        logger.info("entering which_bsd()")
         result, stdout = self.ss.run("uname -r")
         if not result:
             self.close(err_str="failed to determine remote bsd version")
@@ -692,7 +744,7 @@ class SplitCopy:
         """ determines the OpenSSH daemon version
             :returns None:
         """
-        logger.debug("entering which_sshd()")
+        logger.info("entering which_sshd()")
         result, stdout = self.ss.run("sshd -v", exitcode=False)
         if not re.search(r"OpenSSH_", stdout):
             self.close(err_str="failed to determine remote openssh version")
@@ -704,7 +756,7 @@ class SplitCopy:
         """ ensures required binaries exist on remote host
             :returns None:
         """
-        logger.debug("entering req_binaries()")
+        logger.info("entering req_binaries()")
         if not self.junos and not self.evo:
             if self.get_op:
                 req_bins = ["dd", "ls", "df", "rm"]
@@ -720,26 +772,42 @@ class SplitCopy:
                         )
                     )
 
+    def second_elem(self, elem):
+        """ used for key sort
+            :returns: the 2nd part of an element
+        """
+        return elem[1]
+
     def req_sha_binaries(self):
         """ ensures required binaries for sha hash creation exist on remote host
         :returns None:
         """
-        logger.debug("entering req_sha_binaries()")
-        if self.sha_size == 512:
-            sha_bins = ["sha512sum", "sha512", "shasum"]
-        elif self.sha_size == 384:
-            sha_bins = ["sha384sum", "sha384", "shasum"]
-        elif self.sha_size == 256:
-            sha_bins = ["sha256sum", "sha256", "shasum"]
-        elif self.sha_size == 224:
-            sha_bins = ["sha224sum", "sha224", "shasum"]
-        else:
-            sha_bins = ["sha1sum", "sha1", "shasum"]
+        logger.info("entering req_sha_binaries()")
+        sha_bins = []
+        if self.sha_hash.get(512):
+            bins = [("sha512sum", 512), ("sha512", 512), ("shasum", 512)]
+            sha_bins.extend(bins)
+        if self.sha_hash.get(384):
+            bins = [("sha384sum", 384), ("sha384", 384), ("shasum", 384)]
+            sha_bins.extend(bins)
+        if self.sha_hash.get(256):
+            bins = [("sha256sum", 256), ("sha256", 256), ("shasum", 256)]
+            sha_bins.extend(bins)
+        if self.sha_hash.get(224):
+            bins = [("sha224sum", 224), ("sha224", 224), ("shasum", 224)]
+            sha_bins.extend(bins)
+        if self.sha_hash.get(1):
+            bins = [("sha1sum", 1), ("sha1", 1), ("shasum", 1)]
+            sha_bins.extend(bins)
+
+        sha_bins = sorted(set(sha_bins), reverse=True, key=self.second_elem)
+        logger.info(sha_bins)
 
         for req_bin in sha_bins:
-            result, stdout = self.ss.run("which {}".format(req_bin))
+            result, stdout = self.ss.run("which {}".format(req_bin[0]))
             if result:
-                self.sha_bin = req_bin
+                self.sha_bin = req_bin[0]
+                self.sha_len = req_bin[1]
                 break
         if not self.sha_bin:
             self.close(
@@ -785,7 +853,7 @@ class SplitCopy:
         """ concatenates the file chunks into one file on remote host
             :returns None:
         """
-        logger.debug("entering join_files_remote()")
+        logger.info("entering join_files_remote()")
         print("joining files...")
         result = False
         try:
@@ -816,7 +884,7 @@ class SplitCopy:
         """ concatenates the file chunks into one file on local host
             :returns None:
         """
-        logger.debug("entering join_files_local()")
+        logger.info("entering join_files_local()")
         print("joining files...")
         src_files = glob.glob(self.local_tmpdir + os.path.sep + self.remote_file + "*")
         dst_file = self.local_dir + os.path.sep + self.local_file
@@ -836,7 +904,7 @@ class SplitCopy:
             compares against local sha
             :returns None:
         """
-        logger.debug("entering remote_sha_put()")
+        logger.info("entering remote_sha_put()")
         print("generating remote sha hash...")
         result, stdout = self.ss.run(
             "ls {}/{}".format(self.remote_dir, self.remote_file)
@@ -848,14 +916,8 @@ class SplitCopy:
             self.config_rollback = False
             self.close(err_str=err)
 
-        if self.sha_size == 512 and self.sha_bin == "shasum":
-            cmd = "shasum -a 512"
-        elif self.sha_size == 384 and self.sha_bin == "shasum":
-            cmd = "shasum -a 384"
-        elif self.sha_size == 256 and self.sha_bin == "shasum":
-            cmd = "shasum -a 256"
-        elif self.sha_size == 224 and self.sha_bin == "shasum":
-            cmd = "shasum -a 224"
+        if self.sha_bin == "shasum":
+            cmd = "shasum -a {}".format(self.sha_len)
         else:
             cmd = "{}".format(self.sha_bin)
 
@@ -865,16 +927,18 @@ class SplitCopy:
         if not result:
             print(
                 "remote sha hash generation failed or timed out, "
-                'manually check the output of "{} <file>" and '
-                "compare against {}".format(cmd, self.local_sha)
+                'manually check the output of "{} {}/{}" and '
+                "compare against {}".format(
+                    cmd, self.remote_dir, self.remote_file, self.sha_hash[self.sha_len]
+                )
             )
             return
         if re.match(r"sha.*sum", self.sha_bin):
             remote_sha = stdout.split("\n")[1].split()[0].rstrip()
         else:
             remote_sha = stdout.split("\n")[1].split()[3].rstrip()
-        logger.debug("remote sha = {}".format(remote_sha))
-        if self.local_sha == remote_sha:
+        logger.info("remote sha = {}".format(remote_sha))
+        if self.sha_hash[self.sha_len] == remote_sha:
             print(
                 "local and remote sha hash match\nfile has been "
                 "successfully copied to {}:{}/{}".format(
@@ -895,57 +959,60 @@ class SplitCopy:
             the protocol used to copy, FreeBSD and OpenSSH version
             :returns None:
         """
-        logger.debug("entering file_split_size()")
-        if sys.version_info < (3, 6):
-            # if using python < 3.6 the maximum number of simultaneous transfers is 5.
-            self.split_size = int(divmod(self.file_size, 5)[0])
-            return
+        logger.info("entering file_split_size()")
 
-        cpu_count = 1
         try:
-            cpu_count = multiprocessing.cpu_count()
+            cpu_count = os.cpu_count()
         except NotImplementedError:
-            pass
+            cpu_count = 1
+        max_workers = min(32, cpu_count * 5)
 
-        # python 3.6+ allows 5 simultaneous transfers per cpu
-        if cpu_count <= 4:
-            ftp_max, scp_max = cpu_count * 5, cpu_count * 5
+        # each uid can have max of 64 processes
+        # modulate worker count to consume no more than 40 pids
+        if self.copy_proto == "ftp":
+            # ftp creates 1 user process per chunk, no modulation required
+            self.split_size = ceil(self.file_size / max_workers)
+        elif max_workers <= 10:
+            # 1 or 2 cpu cores, 5 or 10 workers will create 20-40 pids
+            # no modulation required
+            self.split_size = ceil(self.file_size / max_workers)
         else:
-            # ftp creates 1 user process per chunk
             # scp to FreeBSD 6 based junos creates 3 user processes per chunk
             # scp to FreeBSD 10+ based junos creates 2 user processes per chunk
             # +1 user process if openssh version is >= 7.4
-            # each uid can have max of 64 processes
-            # values here will leave min 19 processes headroom
-            max_pids = 39
-            ftp_max = max_pids
+            max_pids = 40
             if self.sshd_version >= 7.4 and self.bsd_version == 6.0:
-                scp_pids = 4
-            elif self.sshd_version >= 7.4 and self.bsd_version >= 11.0:
-                scp_pids = 3
+                pid_count = 4
+            elif self.sshd_version >= 7.4 and self.bsd_version >= 10.0:
+                pid_count = 3
             elif self.bsd_version == 6.0:
-                scp_pids = 3
+                pid_count = 3
             elif self.bsd_version >= 10.0:
-                scp_pids = 2
+                pid_count = 2
             elif self.evo:
-                scp_pids = 3
+                pid_count = 3
             else:
-                # be conservative
-                scp_pids = 4
-            scp_max = round(max_pids / scp_pids)
+                pid_count = 4
+            max_workers = round(max_pids / pid_count)
+            self.split_size = ceil(self.file_size / max_workers)
 
-        if self.copy_proto == "ftp":
-            self.split_size = int(divmod(self.file_size, ftp_max)[0])
-        else:
-            self.split_size = int(divmod(self.file_size, scp_max)[0])
-        logger.debug("file split size = {}".format(self.split_size))
+        # concurrent.futures.ThreadPoolExecutor can be a limiting factor
+        # if using python < 3.5.3 the default max_workers is 5.
+        # see https://github.com/python/cpython/blob/v3.5.2/Lib/asyncio/base_events.py
+        # hence defining a custom executor to normalize max_workers across versions
+        self.executor = concurrent.futures.ThreadPoolExecutor(max_workers=max_workers)
+        logger.info(
+            "max_workers = {}, cpu_count = {}, split_size = {}".format(
+                max_workers, cpu_count, self.split_size
+            )
+        )
 
     def split_file_local(self):
         """ splits file into chunks of size already determined in file_split_size()
             This function emulates GNU split.
             :returns None:
         """
-        logger.debug("entering split_file_local()")
+        logger.info("entering split_file_local()")
         print("splitting file...")
         try:
             total_bytes = 0
@@ -956,20 +1023,15 @@ class SplitCopy:
                     with open(
                         "{}{}{}".format(self.local_file, sfx_1, sfx_2), "wb"
                     ) as chunk:
-                        logger.debug(
+                        logger.info(
                             "writing data to {}{}{}".format(
                                 self.local_file, sfx_1, sfx_2
                             )
                         )
-                        chunk_bytes = 0
-                        while chunk_bytes < self.split_size:
-                            data = src.read(_BUF_SIZE_SPLIT)
-                            if data:
-                                chunk.write(data)
-                                chunk_bytes += _BUF_SIZE_SPLIT
-                                total_bytes += _BUF_SIZE_SPLIT
-                            else:
-                                return
+                        src.seek(total_bytes)
+                        data = src.read(self.split_size)
+                        chunk.write(data)
+                        total_bytes += self.split_size
                     if sfx_2 == "z":
                         sfx_1 = "b"
                         sfx_2 = "a"
@@ -986,9 +1048,10 @@ class SplitCopy:
         """ splits file on remote host
             :returns None:
         """
-        logger.debug("entering split_file_remote()")
-        total_blocks = int(self.file_size / _BUF_SIZE)
-        block_size = int(self.split_size / _BUF_SIZE)
+        logger.info("entering split_file_remote()")
+        total_blocks = ceil(self.file_size / _BUF_SIZE)
+        block_size = ceil(self.split_size / _BUF_SIZE)
+        logger.info("total_blocks = {}, block_size = {}".format(total_blocks, block_size))
         cmd = (
             "size_b={}; size_tb={}; i=0; o=00; "
             "while [ $i -lt $size_tb ]; do "
@@ -1013,30 +1076,28 @@ class SplitCopy:
             with SCPClient(transport, **self.copy_kwargs) as scpclient:
                 scpclient.put("split.sh", "{}/split.sh".format(self.remote_tmpdir))
 
-        result, stdout = self.ss.run(
-            "sh {}/split.sh 2>&1".format(self.remote_tmpdir, self.remote_tmpdir)
-        )
-        if not result:
-            err_str = "remote file split operation failed. cmd output was:\n{}".format(
-                stdout
-            )
-            self.close(err_str)
+        self.ss.run("sh {}/split.sh".format(self.remote_tmpdir))
 
     def local_sha_get(self):
         """ generates a sha hash for the combined file on the local host
             :returns None:
         """
-        logger.debug("entering local_sha_get()")
+        logger.info("entering local_sha_get()")
         print("generating local sha hash...")
-        if self.sha_size == 512:
+        if self.sha_hash.get(512):
+            sha_idx = 512
             sha = hashlib.sha512()
-        elif self.sha_size == 384:
+        elif self.sha_hash.get(384):
+            sha_idx = 384
             sha = hashlib.sha384()
-        elif self.sha_size == 256:
+        elif self.sha_hash.get(256):
+            sha_idx = 256
             sha = hashlib.sha256()
-        elif self.sha_size == 224:
+        elif self.sha_hash.get(224):
+            sha_idx = 224
             sha = hashlib.sha224()
         else:
+            sha_idx = 1
             sha = hashlib.sha1()
         dst_file = self.local_dir + os.path.sep + self.local_file
         with open(dst_file, "rb") as dst:
@@ -1045,8 +1106,8 @@ class SplitCopy:
                 sha.update(data)
                 data = dst.read(_BUF_SIZE_READ)
         local_sha = sha.hexdigest()
-        logger.debug("local sha = {}".format(local_sha))
-        if local_sha == self.remote_sha:
+        logger.info("local sha = {}".format(local_sha))
+        if local_sha == self.sha_hash.get(sha_idx):
             print(
                 "local and remote sha hash match\nfile has been "
                 "successfully copied to {}{}{}".format(
@@ -1068,28 +1129,28 @@ class SplitCopy:
             :returns None:
         """
         file_path = self.local_path
-        logger.debug("entering local_sha_put()")
-        if os.path.isfile(file_path + ".sha512"):
-            with open(file_path + ".sha512", "r") as shafile:
-                self.local_sha = shafile.read().split()[0].rstrip()
-                self.sha_size = 512
-        elif os.path.isfile(file_path + ".sha384"):
-            with open(file_path + ".sha384", "r") as shafile:
-                self.local_sha = shafile.read().split()[0].rstrip()
-                self.sha_size = 384
-        elif os.path.isfile(file_path + ".sha256"):
-            with open(file_path + ".sha256", "r") as shafile:
-                self.local_sha = shafile.read().split()[0].rstrip()
-                self.sha_size = 256
-        elif os.path.isfile(file_path + ".sha224"):
-            with open(file_path + ".sha224", "r") as shafile:
-                self.local_sha = shafile.read().split()[0].rstrip()
-                self.sha_size = 224
-        elif os.path.isfile(file_path + ".sha1"):
-            with open(file_path + ".sha1", "r") as shafile:
-                self.local_sha = shafile.read().split()[0].rstrip()
-                self.sha_size = 1
-        else:
+        logger.info("entering local_sha_put()")
+        if os.path.isfile("{}.sha512".format(file_path)):
+            with open("{}.sha512".format(file_path), "r") as shafile:
+                local_sha = shafile.read().split()[0].rstrip()
+                self.sha_hash[512] = local_sha
+        if os.path.isfile("{}.sha384".format(file_path)):
+            with open("{}.sha384".format(file_path), "r") as shafile:
+                local_sha = shafile.read().split()[0].rstrip()
+                self.sha_hash[384] = local_sha
+        if os.path.isfile("{}.sha256".format(file_path)):
+            with open("{}.sha256".format(file_path), "r") as shafile:
+                local_sha = shafile.read().split()[0].rstrip()
+                self.sha_hash[256] = local_sha
+        if os.path.isfile("{}.sha224".format(file_path)):
+            with open("{}.sha224".format(file_path), "r") as shafile:
+                local_sha = shafile.read().split()[0].rstrip()
+                self.sha_hash[224] = local_sha
+        if os.path.isfile("{}.sha1".format(file_path)):
+            with open("{}.sha1".format(file_path), "r") as shafile:
+                local_sha = shafile.read().split()[0].rstrip()
+                self.sha_hash[1] = local_sha
+        if not self.sha_hash:
             print("sha1 not found, generating sha1...")
             sha1 = hashlib.sha1()
             with open(file_path, "rb") as original_file:
@@ -1097,16 +1158,16 @@ class SplitCopy:
                 while data:
                     sha1.update(data)
                     data = original_file.read(_BUF_SIZE_READ)
-            self.local_sha = sha1.hexdigest()
-            self.sha_size = 1
-        logger.debug("local sha hash = {}".format(self.local_sha))
+            local_sha = sha1.hexdigest()
+            self.sha_hash[1] = local_sha
+        logger.info("local sha hashes = {}".format(self.sha_hash))
         self.req_sha_binaries()
 
     def mkdir_remote(self):
         """ creates a tmp directory on the remote host
             :returns None:
         """
-        logger.debug("entering mkdir_remote()")
+        logger.info("entering mkdir_remote()")
         ts = datetime.datetime.strftime(datetime.datetime.now(), "%y%m%d%H%M%S")
         if self.get_op:
             self.remote_tmpdir = "/var/tmp/splitcopy_{}.{}".format(self.remote_file, ts)
@@ -1124,53 +1185,60 @@ class SplitCopy:
         self.rm_remote_tmp = True
 
     def remote_sha_get(self):
-        """ generates a sha hash for the remote file to be copied
+        """ checks for existence of a sha hash file
+            if none found, generates a sha hash for the remote file to be copied
             :returns None:
         """
-        logger.debug("entering remote_sha_get()")
+        logger.info("entering remote_sha_get()")
         result, stdout = self.ss.run("ls -1 {}.sha*".format(self.remote_path))
         if result:
-            sha_file = stdout.split("\n")[1].rstrip()
-            logger.debug("{} file found".format(sha_file))
-            self.sha_size = re.search(r"\.([0-9]+)$", sha_file).group[0]
-            logger.debug("self.sha_size = {}".format(self.sha_size))
-            result, stdout = self.ss.run("cat {}".format(sha_file))
-            if result:
-                self.remote_sha = stdout.split("\n")[1].rstrip()
-            else:
-                logger.debug("unable to read remote sha file")
+            lines = stdout.split("\n")
+            for line in lines:
+                line = line.rstrip()
+                match = re.search(r"\.sha([0-9]+)$", line)
+                if match:
+                    sha_num = int(match.group(1))
+                    logger.info("{} file found".format(line))
+                    result, stdout = self.ss.run("cat {}".format(line))
+                    if result:
+                        self.sha_hash[sha_num] = (
+                            stdout.split("\n")[1].split()[0].rstrip()
+                        )
+                        logger.info("self.sha_hash[{}] added".format(sha_num))
+                    else:
+                        logger.info("unable to read remote sha file {}".format(line))
 
-        if not self.remote_sha:
-            self.sha_size = 1
+        if not self.sha_hash:
+            self.sha_hash[1] = True
             self.req_sha_binaries()
             print("generating remote sha hash...")
-            result, stdout = self.ss.run("{} {}".format(self.sha_bin, self.remote_path))
+            result, stdout = self.ss.run("{} {}".format(self.sha_bin, self.remote_path), timeout=120)
             if not result:
                 self.close(err_str="failed to generate remote sha1")
 
             if re.match(r"sha.*sum", self.sha_bin):
-                self.remote_sha = stdout.split("\n")[1].split()[0].rstrip()
+                self.sha_hash[1] = stdout.split("\n")[1].split()[0].rstrip()
             else:
-                self.remote_sha = stdout.split("\n")[1].split()[3].rstrip()
-        logger.debug("remote sha = {}".format(self.remote_sha))
+                self.sha_hash[1] = stdout.split("\n")[1].split()[3].rstrip()
+        logger.info("remote sha hashes = {}".format(self.sha_hash))
 
     def remote_filesize(self):
         """  determines the remote file size in bytes
             :returns None:
         """
-        logger.debug("entering remote_filesize()")
+        logger.info("entering remote_filesize()")
         result, stdout = self.ss.run("ls -l {}".format(self.remote_path))
         if result:
             self.file_size = int(stdout.split("\n")[1].split()[4])
         else:
             self.close(err_str="cannot determine remote file size")
-        logger.debug("src file size is {}".format(self.file_size))
+        logger.info("src file size is {}".format(self.file_size))
 
     def storage_check_remote(self):
         """ checks whether there is enough storage space on remote node
             :returns None:
         """
-        logger.debug("entering storage_check_remote()")
+        logger.info("entering storage_check_remote()")
         avail_blocks = 0
         print("checking remote storage...")
         if self.get_op:
@@ -1193,7 +1261,7 @@ class SplitCopy:
             self.close(err_str)
 
         avail_bytes = int(avail_blocks) * 1024
-        logger.debug("remote filesystem available bytes is {}".format(avail_bytes))
+        logger.info("remote filesystem available bytes is {}".format(avail_bytes))
         if self.file_size * multiplier > avail_bytes:
             if self.get_op:
                 err_str = (
@@ -1214,11 +1282,11 @@ class SplitCopy:
         """ checks whether there is enough storage space on local node
             :returns None:
         """
-        logger.debug("entering storage_check_local()")
+        logger.info("entering storage_check_local()")
         print("checking local storage...")
         local_tmpdir = tempfile.gettempdir()
         avail_bytes = shutil.disk_usage(local_tmpdir)[2]
-        logger.debug(
+        logger.info(
             "local filesystem {} available bytes is {}".format(
                 local_tmpdir, avail_bytes
             )
@@ -1235,7 +1303,7 @@ class SplitCopy:
 
         if self.get_op:
             avail_bytes = shutil.disk_usage(self.local_dir)[2]
-            logger.debug(
+            logger.info(
                 "local filesystem {} available bytes is {}".format(
                     self.local_dir, avail_bytes
                 )
@@ -1250,15 +1318,18 @@ class SplitCopy:
                 )
                 self.close(err_str)
 
-    def put_files(self, file_name):
+    def put_files(self, sfile):
         """ copies files to remote host via ftp or scp
-            :param file_name: name of the file to copy
-            :type: string
+            :param sfile: name and size of the file to copy
+            :type: list
             :raises TransferError: if file transfer fails 3 times
             :returns None:
         """
         err_count = 0
+        file_name = sfile[0]
+        file_size = sfile[1]
         dstpath = "{}/{}".format(self.remote_tmpdir, file_name)
+        logger.info("{}, size {}".format(file_name, file_size))
         if self.copy_proto == "ftp":
             while err_count < 3:
                 try:
@@ -1298,15 +1369,18 @@ class SplitCopy:
             self.mute = True
             raise TransferError
 
-    def get_files(self, file_name):
+    def get_files(self, sfile):
         """ copies files from remote host via ftp or scp
-            :param file_name: name of the file to copy
-            :type: string
+            :param sfile: name and size of the file to copy
+            :type: list
             :raises TransferError: if file transfer fails 3 times
             :returns None:
         """
         err_count = 0
+        file_name = sfile[0]
+        file_size = sfile[1]
         srcpath = "{}/{}".format(self.remote_tmpdir, file_name)
+        logger.info("{}, size {}".format(file_name, file_size))
         if self.copy_proto == "ftp":
             while err_count < 3:
                 try:
@@ -1368,7 +1442,7 @@ class SplitCopy:
             :returns None:
         """
         self.local_tmpdir = tempfile.mkdtemp()
-        logger.debug(self.local_tmpdir)
+        logger.info(self.local_tmpdir)
 
         def cleanup():
             """ deletes temp dir
@@ -1384,7 +1458,7 @@ class SplitCopy:
             will be deactivated
             :returns None:
         """
-        logger.debug("entering limit_check()")
+        logger.info("entering limit_check()")
         config_stanzas = ["groups", "system services"]
         if self.copy_proto == "ftp":
             limits = [
@@ -1421,7 +1495,7 @@ class SplitCopy:
         # if limits were configured, deactivate them
         if self.command_list:
             print("protocol rate-limit/connection-limit configuration found")
-            logger.debug(self.command_list)
+            logger.info(self.command_list)
             result, stdout = self.ss.run(
                 'cli -c "edit;{}commit and-quit"'.format("".join(self.command_list)),
                 exitcode=False,
@@ -1450,7 +1524,7 @@ class SplitCopy:
         """ revert config change made to remote host
             :returns None:
         """
-        logger.debug("entering limits_rollback()")
+        logger.info("entering limits_rollback()")
         rollback_cmds = "".join(self.command_list)
         rollback_cmds = re.sub("deactivate", "activate", rollback_cmds)
         result, stdout = self.ss.run(
@@ -1476,7 +1550,7 @@ class SplitCopy:
     def remote_cleanup(self, silent=False):
         """ delete tmp directory on remote host
             :param silent: determines whether we announce the dir deletion
-            :type bool:
+            :type: bool
             :returns None:
         """
         if not silent:
